import sys
import time
from selenium import webdriver
from selenium.webdriver.chrome.service import Service
from selenium.webdriver.common.by import By
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.alert import Alert
from selenium.common.exceptions import WebDriverException, TimeoutException

def debug_print_arguments(args):
    print("Arguments received:", args)

def debug_print_url(url):
    print("URL being accessed:", url)

def initialize_webdriver(webdriver_path, browser_width):
    chrome_options = Options()
    chrome_options.add_argument("--headless")
    chrome_options.add_argument(f"--window-size={browser_width}x1080")
    chrome_options.add_argument("--ignore-certificate-errors")
    chrome_options.add_argument("--allow-running-insecure-content")
    service = Service(webdriver_path)
    return webdriver.Chrome(service=service, options=chrome_options)

def handle_basic_auth(driver, basic_auth_id, basic_auth_pass):
    try:
        WebDriverWait(driver, 5).until(EC.alert_is_present())
        alert = Alert(driver)
        alert.send_keys(f"{basic_auth_id}\n{basic_auth_pass}")
        alert.accept()
    except TimeoutException:
        print("The authentication dialog was not displayed.")

def capture_screenshot(driver, file_path, browser_width):
    scroll_height = driver.execute_script("return document.body.scrollHeight")
    driver.set_window_size(browser_width, scroll_height)
    time.sleep(1)
    driver.save_screenshot(file_path)

try:
    debug_print_arguments(sys.argv)
    url = sys.argv[1]
    file_path = sys.argv[2]
    webdriver_path = sys.argv[3]
    browser_width = int(sys.argv[4])
    basic_auth_id = sys.argv[5].encode('shift-jis').decode('utf-8') if len(sys.argv) > 5 else None
    basic_auth_pass = sys.argv[6].encode('shift-jis').decode('utf-8') if len(sys.argv) > 6 else None

    if not url.startswith("http://") and not url.startswith("https://"):
        url = "https://" + url
    debug_print_url(url)

    if basic_auth_id and basic_auth_pass:
        protocol, host = url.split("://")
        url = f"{protocol}://{basic_auth_id}:{basic_auth_pass}@{host}"
    debug_print_url(url)

    driver = initialize_webdriver(webdriver_path, browser_width)
    try:
        driver.get(url)
        if basic_auth_id and basic_auth_pass:
            handle_basic_auth(driver, basic_auth_id, basic_auth_pass)
        WebDriverWait(driver, 5).until(EC.presence_of_element_located((By.TAG_NAME, 'body')))
        capture_screenshot(driver, file_path, browser_width)
        print(file_path)
    except (WebDriverException, TimeoutException) as e:
        print(f"Err:An error has occurred: {str(e)}")
    finally:
        driver.quit()
except IndexError:
    print("Err:how to use: python script.py <url> <file_path> <webdriver_path> <browser_width> [<basic_auth_id> <basic_auth_pass>]")
except ValueError:
    print("Err:browser_width must be an integer.")
except Exception as e:
    print(f"Err:An Unexpected Error has occurred: {str(e)}")
